unit PHList;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Menus, ExtCtrls, ODBF, ODBFTool, Jpeg,
  PHSelect, PHTools, PHCprt;

type
  TListForm = class(TForm)
    GroupBoxFilter: TGroupBox;
    MainMenu: TMainMenu;
    NFile: TMenuItem;
    NPack: TMenuItem;
    NEditDel: TMenuItem;
    NExit: TMenuItem;
    NEdit: TMenuItem;
    NAdd: TMenuItem;
    NModify: TMenuItem;
    NDelete: TMenuItem;
    CheckBoxName: TCheckBox;
    CheckBoxYear: TCheckBox;
    CheckBoxRemark: TCheckBox;
    EditName: TEdit;
    EditYear1: TEdit;
    EditRemark: TEdit;
    LabelAnd: TLabel;
    EditYear2: TEdit;
    OKBtn: TButton;
    GroupBoxList: TGroupBox;
    ListBox: TListBox;
    GroupBoxPhoto: TGroupBox;
    Image: TImage;
    Memo: TMemo;
    DBFTable: TDBFTable;
    DBFStructure: TDBFStructure;
    AddBtn: TButton;
    ModifyBtn: TButton;
    DeleteBtn: TButton;
    NView: TMenuItem;
    NShowdeleted: TMenuItem;
    SaveBtn: TButton;
    LabelYear: TLabel;
    SaveDialog: TSaveDialog;
    UndeleteBtn: TButton;
    LabelDeleted: TLabel;
    NHelp: TMenuItem;
    NAbout: TMenuItem;
    NUndelete: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure NShowdeletedClick(Sender: TObject);
    procedure AddBtnClick(Sender: TObject);
    procedure ModifyBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure UndeleteBtnClick(Sender: TObject);
    procedure NExitClick(Sender: TObject);
    procedure NPackClick(Sender: TObject);
    procedure ListBoxClick(Sender: TObject);
    procedure SaveBtnClick(Sender: TObject);
    procedure NAboutClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ClearList;
    procedure UpdateList;
    procedure EditRecord (RecNo: Longint);
    function CheckStructure: Boolean;
  end;

var
  ListForm: TListForm;

implementation

{$R *.DFM}

procedure TListForm.ClearList;
var Index: Integer;
    IValue: TIntValue;
begin
  for Index:=0 to ListBox.Items.Count-1 do
    begin
      IValue:=TIntValue(ListBox.Items.Objects[Index]);
      if IValue<>NIL
         then IValue.Free;
    end;
  ListBox.Clear;
end;

procedure TListForm.UpdateList;
var Index, Count, StoredIndex: Longint;
    DBFFieldName, DBFFieldYear, DBFFieldRemark: TDBFField;
    RecordVisible: Boolean;
    Filter, Data: String;
    Year1, Year2, YearData: Integer;
begin
  if ListBox.Items.Count>0
     then StoredIndex:=ListBox.ItemIndex
     else StoredIndex:=-1;
  if StoredIndex>=0
     then StoredIndex:=TIntValue(ListBox.Items.Objects[StoredIndex]).Value;
  ClearList;
  Image.Picture.Assign(NIL);
  Memo.Clear;
  LabelDeleted.Caption:='';
  LabelYear.Caption:='';
  Count:=DBFTable.RecordCount;
  DBFFieldName:=DBFTable.FieldByName('NAME');
  DBFFieldYear:=DBFTable.FieldByName('YEAR');
  DBFFieldRemark:=DBFTable.FieldByName('REMARK');
  for Index:=1 to Count do
    begin
      try
         DBFTable.Go(Index);
      except
         on E: Exception do
            begin
              MessageDlg(E.Message,mtError,[mbOK],0);
              break;
            end;
      end;
      RecordVisible:=(not DBFTable.Deleted) OR (NShowdeleted.Checked);
      if RecordVisible then
         if (CheckBoxName.Checked) AND (EditName.Text<>'') then
            begin
              Filter:=UpperCase(EditName.Text);
              Data:=UpperCase(DBFFieldName.Text);
              RecordVisible:=RecordVisible AND (Pos(Filter,Data)>=1);
            end;
      if RecordVisible then
         if CheckBoxYear.Checked then
            begin
              try
                 Year1:=StrToInt(EditYear1.Text);
              except
                 Year1:=0;
              end;
              try
                 Year2:=StrToInt(EditYear2.Text);
              except
                 Year2:=9999;
              end;
              YearData:=Round(DBFFieldYear.AsFloat);
              RecordVisible:=RecordVisible AND
                (YearData>=Year1) AND (YearData<=Year2);
            end;
      if RecordVisible then
         if (CheckBoxRemark.Checked) AND (EditRemark.Text<>'') then
            begin
              Filter:=UpperCase(EditRemark.Text);
              Data:=UpperCase(DBFFieldRemark.Text);
              RecordVisible:=RecordVisible AND (Pos(Filter,Data)>=1);
            end;
      if RecordVisible
         then ListBox.Items.AddObject(DBFFieldName.Text,TIntValue.Create(Index));
    end;
  Count:=ListBox.Items.Count;
  for Index:=0 to Count-1 do
    if StoredIndex=TIntValue(ListBox.Items.Objects[Index]).Value then
       begin
         ListBox.ItemIndex:=Index;
         ListBoxClick(Self);
         break;
       end;
end;

procedure TListForm.EditRecord (RecNo: Longint);
var Index, Size: Longint;
    DBFField: TDBFField;
    MS: TMemoryStream;
    Buffer: Pointer;
begin
  RecordForm:=TRecordForm.Create(NIL);
  try
     if RecNo>0 then
        begin
          DBFTable.RecNo:=RecNo;
          RecordForm.CheckBoxDeleted.Checked:=DBFTable.Deleted;
          DBFField:=DBFTable.FieldByName('NAME');
          RecordForm.EditName.Text:=DBFField.AsString;
          DBFField:=DBFTable.FieldByName('YEAR');
          RecordForm.EditYear.Text:=DBFField.AsString;
          DBFField:=DBFTable.FieldByName('REMARK');
          RecordForm.EditRemark.Text:=DBFField.AsString;
          DBFField:=DBFTable.FieldByName('IMAGE');
          if DBFField.Defined then
             begin
               Size:=DBFField.GetBlobToBuffer(Buffer,0);
               MS:=TMemoryStream.Create;
               try
                  MS.SetSize(Size);
                  DBFField.GetBlobToBuffer(MS.Memory,Size);
                  RecordForm.JPEGImage.LoadFromStream(MS);
                  RecordForm.ShowImage;
               finally
                  MS.Free;
               end;
             end
          else RecordForm.ClearImage;
        end;
     if RecordForm.ShowModal=mrOK then
        begin
          if RecNo<=0
             then DBFTable.AppendBlank;
          if RecordForm.CheckBoxDeleted.Checked
             then DBFTable.Delete
             else DBFTable.Undelete;
          DBFField:=DBFTable.FieldByName('NAME');
          DBFField.AsString:=RecordForm.EditName.Text;
          DBFField:=DBFTable.FieldByName('YEAR');
          try
             DBFField.AsFloat:=StrToInt(RecordForm.EditYear.Text);
          except
             DBFField.ClearValue;
          end;
          DBFField:=DBFTable.FieldByName('REMARK');
          DBFField.AsString:=RecordForm.EditRemark.Text;
          DBFField:=DBFTable.FieldByName('IMAGE');
          if RecordForm.ImageChanged then
             if RecordForm.Image.Picture.Graphic=NIL
                then DBFField.ClearValue
                else
                  begin
                    MS:=TMemoryStream.Create;
                    try
                       RecordForm.JPEGImage.SaveToStream(MS);
                       DBFField.SetBlobFromBuffer(MS.Memory,MS.Size);
                    finally
                       MS.Free;
                    end;
                  end;
          DBFTable.ReplaceRecord;
        end;
  finally
     RecordForm.Free;
     UpdateList;
  end;
end;

function TListForm.CheckStructure: Boolean;
var Index: Integer;
    DBFFieldDef1, DBFFieldDef2: TDBFFieldDef;
begin
  Result:=FALSE;
  if DBFTable.FieldCount<>DBFStructure.FieldCount
     then exit;
  for Index:=0 to DBFTable.FieldCount-1 do
    begin
      DBFFieldDef1:=DBFTable.FieldDefs.Items[Index];
      DBFFieldDef2:=DBFStructure.Items[Index];
      if DBFFieldDef1.Description<>DBFFieldDef2.Description
         then exit;
    end;
  Result:=TRUE;
end;

procedure TListForm.FormCreate(Sender: TObject);
var Index: Integer;
    DBFName: String;
begin
  DBFName:=Application.ExeName;
  for Index:=Length(DBFName) downto 1 do
    if DBFName[Index]='\' then
       begin
         DBFName:=Copy(DBFName,1,Index);
         break;
       end;
  DBFName:=DBFName+'PHOTOS.DBF';
  try
     DBFTable.Use(DBFName);
     if not CheckStructure then
        begin
          MessageDlg('Table '+DBFName+' has different structure. Remove it '+
            'from this directory and run the program again. ',mtInformation,
            [mbOK],0);
          GroupBoxFilter.Enabled:=FALSE;
          GroupBoxList.Enabled:=FALSE;
          GroupBoxPhoto.Enabled:=FALSE;
          NEdit.Enabled:=FALSE;
          NView.Enabled:=FALSE;
          exit;
        end;
  except
     if MessageDlg('Table '+DBFName+' can`t be opened. Create the table now? ',
        mtInformation,[mbYes,mbNo],0)=mrYes
        then DBFTable.CreateTable(DBFName,DBFStructure)
     else
        begin
          GroupBoxFilter.Enabled:=FALSE;
          GroupBoxList.Enabled:=FALSE;
          GroupBoxPhoto.Enabled:=FALSE;
          NEdit.Enabled:=FALSE;
          NView.Enabled:=FALSE;
          exit;
        end;
  end;
  UpdateList;
end;

procedure TListForm.FormDestroy(Sender: TObject);
begin
  ClearList;
  DBFTable.CloseTable;
end;

procedure TListForm.OKBtnClick(Sender: TObject);
begin
  UpdateList;
end;

procedure TListForm.NShowdeletedClick(Sender: TObject);
begin
  NShowdeleted.Checked:=Not NShowdeleted.Checked;
  UpdateList;
end;

procedure TListForm.AddBtnClick(Sender: TObject);
begin
  EditRecord(0);
end;

procedure TListForm.ModifyBtnClick(Sender: TObject);
var Index: Integer;
begin
  Index:=ListBox.ItemIndex;
  if Index>=0
     then EditRecord(TIntValue(ListBox.Items.Objects[Index]).Value);
end;

procedure TListForm.DeleteBtnClick(Sender: TObject);
var Index: Integer;
begin
  Index:=ListBox.ItemIndex;
  if Index>=0 then
     begin
       DBFTable.Go(TIntValue(ListBox.Items.Objects[Index]).Value);
       if Not DBFTable.Deleted then
          begin
            DBFTable.Delete;
            DBFTable.ReplaceRecord;
            UpdateList;
          end;
     end;
end;

procedure TListForm.UndeleteBtnClick(Sender: TObject);
var Index: Integer;
begin
  Index:=ListBox.ItemIndex;
  if Index>=0 then
     begin
       DBFTable.Go(TIntValue(ListBox.Items.Objects[Index]).Value);
       if DBFTable.Deleted then
          begin
            DBFTable.Undelete;
            DBFTable.ReplaceRecord;
            UpdateList;
          end;
     end;
end;

procedure TListForm.NExitClick(Sender: TObject);
begin
  Close;
end;

procedure TListForm.NPackClick(Sender: TObject);
begin
  DBFTable.Pack;
  DBFTable.PackMemo;
  UpdateList;
end;

procedure TListForm.ListBoxClick(Sender: TObject);
var Index, Size: Integer;
    DBFField: TDBFField;
    JPEGImage: TJpegImage;
    MS: TMemoryStream;
    Buffer: Pointer;
    W1, H1, W2, H2: Integer;
begin
  Index:=ListBox.ItemIndex;
  if Index<0
     then exit;
  DBFTable.Go(TIntValue(ListBox.Items.Objects[Index]).Value);
  if DBFTable.Deleted
     then LabelDeleted.Caption:='Deleted'
     else LabelDeleted.Caption:='';
  DBFField:=DBFTable.FieldByName('YEAR');
  LabelYear.Caption:=DBFField.Text;
  DBFField:=DBFTable.FieldByName('REMARK');
  Memo.Lines.Clear;
  Memo.Lines[0]:=DBFField.Text;
  DBFField:=DBFTable.FieldByName('IMAGE');
  Image.Picture.Assign(NIL);
  if DBFField.Defined then
     begin
       Size:=DBFField.GetBlobToBuffer(Buffer,0);
       MS:=TMemoryStream.Create;
       JpegImage:=TJpegImage.Create;
       try
          MS.SetSize(Size);
          DBFField.GetBlobToBuffer(MS.Memory,Size);
          JPEGImage.LoadFromStream(MS);
          Image.Picture.Assign(JPEGImage);
          if Not Image.Picture.Graphic.Empty then
             begin
               W1:=JPEGImage.Width;
               H1:=JPEGImage.Height;
               if (W1>200) OR (H1>200) then
                  begin
                    Image.Stretch:=TRUE;
                    if W1>H1 then
                       begin
                         W2:=200;
                         H2:=Round(H1*W2/W1);
                       end
                    else
                       begin
                         H2:=200;
                         W2:=Round(W1*H2/H1);
                       end;
                    Image.Left:=8 + (200-W2) div 2;
                    Image.Top:=16 + (200-H2) div 2;
                    Image.Width:=W2;
                    Image.Height:=H2;
                  end
               else
                  begin
                    Image.Stretch:=FALSE;
                    Image.Left:=8;
                    Image.Top:=16;
                    Image.Width:=200;
                    Image.Height:=200;
                  end;
             end;
       finally
          MS.Free;
          JpegImage.Free;
       end;
     end;
end;

procedure TListForm.SaveBtnClick(Sender: TObject);
var Index: Integer;
    DBFField: TDBFField;
    PictName: String;
    Allow: Boolean;
begin
  Index:=ListBox.ItemIndex;
  if Index<0
     then exit;
  DBFTable.Go(TIntValue(ListBox.Items.Objects[Index]).Value);
  DBFField:=DBFTable.FieldByName('IMAGE');
  if DBFField.Defined then
     if SaveDialog.Execute then
        begin
          PictName:=SaveDialog.FileName;
          Allow:=TRUE;
          if FileExists(PictName) then
             if MessageDlg('File '+PictName+' already exists. Overwrite?',
                mtWarning, [mbYes, mbNo], 0)<>mrYes
                then Allow:=FALSE;
          if Allow
             then DBFField.GetBlobToFile(PictName);
        end;
end;

procedure TListForm.NAboutClick(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

end.

